@@ -1003,3 +1003,35 @@ def on_cancel(submission_uuid):
         )
         logger.exception(error_message)
         raise PeerAssessmentInternalError(error_message) from ex
+
+
+def get_waiting_step_details(
+    course_id,
+    item_id,
+    submission_uuids,
+    must_be_graded_by
+):
+    """
+    Proxy method to `get_waiting_step_details` model method.
+    Retrieves information about users in the waiting step (waiting for peer reviews).
+
+    Args:
+        course_id (str): The course that this problem belongs to.
+        item_id (str): The student_item (problem) that we want to know statistics about.
+        submission_uuids (list): A list of submission UUIDs to filter the results for,
+                                    if None is given, this will return all students which
+                                    the peer step is not complete.
+        must_be_graded_by (int): number of required peer reviews for this problem.
+
+    Returns:
+        dict: a dictionary that contains information about students in the waiting step.
+              The dictionary includes the following information: `student_id`, `created_at` (
+              timestamp of when the step was created), `graded` (how many peers the student
+              graded) and `graded_by` (how many peers graded this student).
+    """
+    return PeerWorkflow.get_waiting_step_details(
+        course_id,
+        item_id,
+        submission_uuids,
+        must_be_graded_by
+    )
